#ifndef VPP_STATISTICS_INTERNAL_H
#define VPP_STATISTICS_INTERNAL_H 1

#define VPP_API_CON_NAME_LEN    32
#define API_RX_Q_SIZE 2048
#define VPP_STAT_SOCKET_FILE "/run/vpp/stats.sock"
#define VPP_STAT_IF_MAX_COUNT 512
#define VPP_STAT_IF_NAME_LEN  32
#define VPP_STAT_IF_NAME_UUID_LEN  256
#define HOST_IF_NAME_SIZE 64
#define VHOST_SOCKET_FILENAME 256

typedef u64 cdtime_t;

#define NS_TO_CDTIME_T(ns)                                                     \
  (cdtime_t) {                                                                 \
    ((((cdtime_t)(ns)) / 1000000000) << 30) |                                  \
        ((((((cdtime_t)(ns)) % 1000000000) << 30) + 500000000) / 1000000000)   \
  }
  
#define TIMESPEC_TO_CDTIME_T(ts)                                               \
  NS_TO_CDTIME_T(1000000000ULL * (ts)->tv_sec + (ts)->tv_nsec)



/* typedef unsigned long long counter_t; */
typedef double gauge_t;
typedef i64 derive_t;
typedef u64 absolute_t;

union value_u {
  unsigned long long counter;
  double gauge;
  i64 derive;
  u64 absolute;
};
typedef union value_u value_t;

typedef struct vpp_api_connection_s {
    int api_connected;
    char api_connection_name[VPP_API_CON_NAME_LEN];
    unix_shared_memory_queue_t *api_vl_input_queue;
    u32 api_client_index;
    volatile int api_result_ready;
    volatile int api_result_retval;
    clib_time_t api_clib_time;
    
    vlib_main_t vlib_global_main;
    vlib_main_t **vlib_mains;
} vpp_api_connection_t;

typedef struct vpp_if_mapping_db
{
    int       sw_if_index;
    char      if_name[VPP_STAT_IF_NAME_LEN];
    i64       packets_rx;
    i64       packets_tx;
    i64       octets_rx;
    i64       octets_tx;
    i64       drops;
    i64       errors_rx;
    i64       errors_tx;
    char      if_uuid[VPP_STAT_IF_NAME_UUID_LEN];
}vpp_if_mapping_db_t;

typedef struct if_rate_db_s
{
	  value_t		last_rx_bytes_count;
	  value_t		last_tx_bytes_count;
   value_t		last_rx_packet_count;
	  value_t		last_tx_packet_count;
	  cdtime_t 	last_time; /* stores the time when counters above are saved */
}if_rate_db_t;

typedef struct vpp_to_host_if_map_s
{
    int sw_if_index;
    int host_if_index;
    char host_if_name[HOST_IF_NAME_SIZE]; /* stores host ppp name (pppX) and vxlan name */
    int is_vxlan;
}vpp_to_host_if_map_t;

typedef struct vpp_to_vhost_if_map_s
{
    int sw_if_index;
    char interface_name[HOST_IF_NAME_SIZE];
    char sock_filename[VHOST_SOCKET_FILENAME];
}vpp_to_vhost_if_map_t;

typedef u64 cdtime_t;

#define NS_TO_CDTIME_T(ns)                                                     \
  (cdtime_t) {                                                                 \
    ((((cdtime_t)(ns)) / 1000000000) << 30) |                                  \
        ((((((cdtime_t)(ns)) % 1000000000) << 30) + 500000000) / 1000000000)   \
  }
  
#define TIMESPEC_TO_CDTIME_T(ts)                                               \
  NS_TO_CDTIME_T(1000000000ULL * (ts)->tv_sec + (ts)->tv_nsec)

#endif /* VPP_STATISTICS_INTERNAL_H */
